#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <grp.h>
#include <pwd.h>
#include <string.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

static GtkWidget *entry_value = NULL;
static GtkCList *clist_value = NULL;
static GtkWidget *window = NULL;

static char ftp_options[1024];

struct group *ggid;
struct passwd *uuid;

/* Set options for PureFTPD GUI */
void
set_options                    (char	         *filename,
				gpointer         user_data)
{
   printf("va falloir parser %s\n", filename);
}

/* Get all the options for the GUI configuration tool and store them in a string */
void
get_options                  (gpointer         user_data)
{
   char *ip_addr;
   int portnum;
   int idle_time;
   int pasv_range_from;
   int pasv_range_to;
   int dos_max_files;
   int dos_max_depth;
   int cpu_load;
   char *fortune;
   char *log_facilities;
   int max_users;
   int max_users_same_ip;
   char *ldap_conf;
   int bw_anon_only;
   int bw_everyone;
   int ratio_ul;
   int ratio_dl;
   
   strcpy(ftp_options, " ");  // Initialize the string

   // Standalone Option
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"S_checkbutton"))->active)
	{
	   strcat(ftp_options, "-S");
	   // IP Address
	   ip_addr = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(user_data),"S_ip_entry")));
	   if (ip_addr != NULL)
	     strcat(ftp_options, (gpointer) _(g_strdup_printf(" %s", ip_addr)));
	     	   
	   // Port Number
	   portnum = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"S_port_spinbutton")));
	   if (portnum != 21)
	     strcat(ftp_options, (gpointer) _(g_strdup_printf(" %d", portnum)));
	     
	}
   
   // Idle-Time
   idle_time = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"I_spinbutton")));
   if (idle_time != 10)
     strcat(ftp_options, (gpointer) _(g_strdup_printf(" -I %d", idle_time)));
      
   // Do not resolve hostnames
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"H_checkbutton"))->active)
	strcat(ftp_options, " -H");

   // Compatibility hacks
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"b_checkbutton"))->active)
	strcat(ftp_options, " -b");

   // Passive Range
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"p_checkbutton"))->active)
     {
	pasv_range_from =gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"p_rangefrom_spinbutton")));
	pasv_range_to =gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"p_rangeto_spinbutton")));
	strcat(ftp_options, (gpointer) _(g_strdup_printf(" -p %d %d", pasv_range_from, pasv_range_to)));
     }
   
   // Force this IP in reply
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"P_checkbutton"))->active)
     {
	ip_addr = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(user_data),"P_entry")));
	if (ip_addr != NULL )
	  strcat(ftp_options, (gpointer) _(g_strdup_printf(" -P %s", ip_addr)));
	/* TODO :
	 * if IP is missing, get the default current system inet IP address */
	else
	  strcat(ftp_options, " -P 127.0.0.1");
     }
   
   // DoS Protection
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"L_checkbutton"))->active)
     {
	dos_max_files =gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"Lf_spinbutton")));
	dos_max_depth =gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"Ld_spinbutton")));
	strcat(ftp_options, (gpointer) _(g_strdup_printf(" -L %d %d", dos_max_files, dos_max_depth)));
     }

   // Anonymous transfers not allowed on CPU load too high
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"m_checkbutton"))->active)
     {
	cpu_load =gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"m_spinbutton")));
	strcat(ftp_options, (gpointer) _(g_strdup_printf(" -m %d", cpu_load)));
     }

   // FXP Options
   // FXP for non-anonymous users
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"w_checkbutton"))->active)
	strcat(ftp_options, " -w");
   // FXP for everyone
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"W_checkbutton"))->active)
	strcat(ftp_options, " -W");
   
   // Fortune display
   fortune = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(user_data),"fortune_file_entry")));
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"F_checkbutton"))->active)
        strcat(ftp_options, (gpointer) _(g_strdup_printf(" -F %s", fortune)));
   
   // Logging Options
   // Turn on debug mode
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"d_checkbutton"))->active)
	{
	   strcat(ftp_options, " -d");
	   // Logging facilities
	   log_facilities = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(user_data),"f_combo_entry")));
	   strcat(ftp_options, (gpointer) _(g_strdup_printf(" %s", log_facilities)));
	}
   
   
   // Max users connected
   max_users = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"c_spinbutton"))); 
   if (max_users != 0)
     strcat(ftp_options, (gpointer) _(g_strdup_printf(" -c %d", max_users)));
   
   // Max users connected from the same ip
   max_users_same_ip = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"C_spinbutton")));
   if (max_users_same_ip != 0)
     strcat(ftp_options, (gpointer) _(g_strdup_printf(" -C %d", max_users_same_ip)));

   // LDAP config file
   ldap_conf=gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(user_data),"ldap_file_entry")));
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"l_checkbutton"))->active)
     strcat(ftp_options, (gpointer) _(g_strdup_printf(" -l %s", ldap_conf)));
   
   // chroot everyone but root
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"A_checkbutton"))->active)
	strcat(ftp_options, " -A");
   
   // Users can't modify files beginnig with "."
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"x_checkbutton"))->active)
	strcat(ftp_options, " -x");
   
   // Users can't even read files beginning with a "."
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"X_checkbutton"))->active)
	strcat(ftp_options, " -X");
   
   
   
   
   
   
   // Anonymous access management
   // Only allow anonymous connection 
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"e_checkbutton"))->active)
	strcat(ftp_options, " -e");
   
   // Anonymous users can create dir.
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"M_checkbutton"))->active)
	strcat(ftp_options, " -M");
   
   // Anonymous cannot dowload files owned by "ftp" user
      if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"s_checkbutton"))->active)
	strcat(ftp_options, " -s");
    
   // Bandwidth Management
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"bw_checkbutton"))->active)
     {
	bw_anon_only = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"t_spinbutton")));
	bw_everyone = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"T_spinbutton")));
	
	if (bw_anon_only !=0)
	  strcat(ftp_options, (gpointer) _(g_strdup_printf(" -t %d", bw_anon_only)));
	if (bw_everyone !=0)
	  strcat(ftp_options, (gpointer) _(g_strdup_printf(" -T %d", bw_everyone)));
     }
   
   // Ratio Management
   if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"Q_checkbutton"))->active) 
     {
	ratio_ul = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"q_u_spinbutton")));
	ratio_dl = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"q_d_spinbutton")));
	
	if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"anon_radiobutton"))->active)
	  strcat(ftp_options, (gpointer) _(g_strdup_printf(" -q %d %d", ratio_ul, ratio_dl )));
	if (GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(user_data),"everyone_radiobutton"))->active)
	  strcat(ftp_options, (gpointer) _(g_strdup_printf(" -Q %d %d", ratio_ul, ratio_dl )));
     }
  
}


void
browse_file                            (GtkButton       *button,
                                        gpointer         user_data)
{
   GtkFileSelection *fileselect;
   entry_value = GTK_WIDGET(user_data);
   
   fileselect = (GtkFileSelection *)gtk_file_selection_new("Browse...");
   
   
   gtk_signal_connect (GTK_OBJECT (fileselect->ok_button), "clicked",
		       GTK_SIGNAL_FUNC (file_select),
		       GTK_OBJECT(fileselect));
   
   gtk_signal_connect_object(GTK_OBJECT (fileselect->cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT(fileselect));
   
   gtk_signal_connect(GTK_OBJECT(fileselect), "destroy",
		      GTK_SIGNAL_FUNC(gtk_widget_destroy),
		      NULL);
   
   gtk_window_position (GTK_WINDOW (fileselect), GTK_WIN_POS_MOUSE);
					 
   gtk_widget_show (GTK_WIDGET (fileselect));
   gdk_window_raise (GTK_WIDGET (fileselect)->window);

   
}

void 
file_select                       (GtkWidget       *widget,
				   gpointer        user_data)
{
   gchar * filename;
   
   filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data));
   gtk_entry_set_text(GTK_ENTRY (entry_value ), filename);
   
   gtk_widget_destroy(GTK_WIDGET(user_data));
   
}


void
get_clist_name                         (GtkButton       *button,
                                        gpointer         user_data)
{
   clist_value = GTK_CLIST (user_data);
}


void
delete_item                           (GtkButton       *button,
                                        gpointer         user_data)
{
   GList *sel;
   clist_value = GTK_CLIST (user_data);
      
   sel = GTK_CLIST(clist_value)->selection;
   if (sel !=NULL)
     gtk_clist_remove(GTK_CLIST(clist_value), (int) sel->data);
}




void
add_name_u_clist                       (GtkButton       *button,
                                        gpointer         user_data)
{
   gchar *uid[2];
   gchar *checkpoint;
   entry_value = GTK_WIDGET (user_data);
  
   
   checkpoint = gtk_entry_get_text(GTK_ENTRY(entry_value));
   
   
   uuid = getpwnam(checkpoint);
   
   uid[0] = g_strdup_printf("%d\n", uuid->pw_uid);
   uid[1] = checkpoint;
   gtk_clist_append (GTK_CLIST(clist_value), uid);
}


void
add_name_a_clist                       (GtkButton       *button,
                                        gpointer         user_data)
{
   gchar *gid[2];
   entry_value = GTK_WIDGET (user_data);
  
   ggid = getgrnam(gtk_entry_get_text(GTK_ENTRY(entry_value)));
   
   gid[0] = g_strdup_printf("%d\n", ggid->gr_gid);
   gid[1] = gtk_entry_get_text(GTK_ENTRY(entry_value));
  
   
   gtk_clist_append (GTK_CLIST(clist_value), gid);   
}


void
on_save_menu_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   FILE *pureguirc;
   char *user_home; 
   char path_to_file[128];
   get_options(user_data);
   
   user_home = getenv("HOME");
   strcpy(path_to_file, user_home);
   strcat(path_to_file, "/.pureguirc");
   // Now we have the full path to the file, let's open it
   pureguirc = fopen(path_to_file, "w+");
   if (pureguirc) 
     {
	fwrite(ftp_options,
	       strlen(ftp_options), 1,
	       pureguirc);
	fclose(pureguirc);
     }
}

void 
load_rc                           (gpointer user_data)
{
   char *user_home; 
   char filename[128];
     
   user_home = getenv("HOME");
   strcpy(filename, user_home);
   strcat(filename, "/.pureguirc");
   set_options(filename, user_data);
}


void
on_load_from_file_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   GtkFileSelection *load_from_fs;
   if (window==NULL)
     window = GTK_WIDGET(user_data);

   load_from_fs = GTK_FILE_SELECTION(gtk_file_selection_new("Browse..."));
   gtk_signal_connect (GTK_OBJECT (load_from_fs->ok_button), "clicked",
		       GTK_SIGNAL_FUNC (file_load_from),
		       GTK_OBJECT(load_from_fs));
   gtk_signal_connect_object(GTK_OBJECT (load_from_fs->cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT(load_from_fs));
   gtk_signal_connect(GTK_OBJECT(load_from_fs), "destroy",
		      GTK_SIGNAL_FUNC(gtk_widget_destroy),
		      NULL);
   gtk_window_position (GTK_WINDOW (load_from_fs), GTK_WIN_POS_MOUSE);
   gtk_widget_show (GTK_WIDGET (load_from_fs));
   gdk_window_raise (GTK_WIDGET (load_from_fs)->window);
}

void
file_load_from                       (GtkWidget       *widget,
				      gpointer        user_data)
{
   gchar *load_from_file;
   
   load_from_file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data));
   set_options(load_from_file, window); 
   gtk_widget_destroy(GTK_WIDGET(user_data));
}


void
on_save_as_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   GtkFileSelection *save_as_fs;
   if (window==NULL)
     window = GTK_WIDGET(user_data);

   save_as_fs = GTK_FILE_SELECTION(gtk_file_selection_new("Browse..."));
   gtk_signal_connect (GTK_OBJECT (save_as_fs->ok_button), "clicked",
		       GTK_SIGNAL_FUNC (file_save_as),
		       GTK_OBJECT(save_as_fs));
   gtk_signal_connect_object(GTK_OBJECT (save_as_fs->cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT(save_as_fs));
   gtk_signal_connect(GTK_OBJECT(save_as_fs), "destroy",
		      GTK_SIGNAL_FUNC(gtk_widget_destroy),
		      NULL);
   gtk_window_position (GTK_WINDOW (save_as_fs), GTK_WIN_POS_MOUSE);
   gtk_widget_show (GTK_WIDGET (save_as_fs));
   gdk_window_raise (GTK_WIDGET (save_as_fs)->window);
}

void 
file_save_as                       (GtkWidget       *widget,
				   gpointer        user_data)
{
   FILE *pureguirc;
   gchar *save_as_filepath;
   
   save_as_filepath = gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data));
   get_options(window);
   
   pureguirc = fopen(save_as_filepath, "w+");
   if (pureguirc) 
     {
	fwrite(ftp_options,
	       strlen(ftp_options), 1,
	       pureguirc);
	fclose(pureguirc);
     }
   
   
   gtk_widget_destroy(GTK_WIDGET(user_data));
}


void
on_start_pureftpd_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_stop_pureftpd_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_restart_pureftpd_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_pureftpd_gui_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   GtkWidget *About;
   About = create_AboutWindow();
   gtk_widget_show(About);
}

void
on_about_close_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
   gtk_widget_destroy(GTK_WIDGET(user_data));
}

